@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  PDF Translator
  * Purpose:  Definition of classes for OGR .pdf driver.
@@ -15,23 +14,7 @@
  ******************************************************************************
  * Copyright (c) 2010-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #ifndef GDAL_PDF_H_INCLUDED
@@ -52,7 +35,7 @@
 #include <bitset>  // For detecting usage of PDF library
 #include <algorithm>
 
-#include "pdfsdk_headers.h"
+#include "pdfsdk_headers_all.h"
 
 #include "pdfdrivercore.h"
 
@@ -61,7 +44,7 @@
 #include "gdal_pam.h"
 #include "ogrsf_frmts.h"
 
-#include "ogr_mem.h"
+#include "memdataset.h"
 #include "pdfobject.h"
 
 #define PDFLIB_POPPLER 0
@@ -83,6 +66,8 @@ class OGRPDFLayer final : public OGRMemLayer
     int bGeomTypeSet;
     int bGeomTypeMixed;
 
+    CPL_DISALLOW_COPY_ASSIGN(OGRPDFLayer)
+
   public:
     OGRPDFLayer(PDFDataset *poDS, const char *pszName,
                 OGRSpatialReference *poSRS, OGRwkbGeometryType eGeomType);
@@ -106,6 +91,8 @@ class OGRPDFWritableLayer final : public OGRMemLayer
 {
     PDFWritableVectorDataset *poDS;
 
+    CPL_DISALLOW_COPY_ASSIGN(OGRPDFWritableLayer)
+
   public:
     OGRPDFWritableLayer(PDFWritableVectorDataset *poDS, const char *pszName,
                         OGRSpatialReference *poSRS,
@@ -161,13 +148,13 @@ typedef std::map<int, TPdfiumPageStruct *> TMapPdfiumPages;
 /************************************************************************/
 
 // Structure for Mutex on File
-typedef struct
+struct TPdfiumDocumentStruct
 {
-    char *filename;
-    CPDF_Document *doc;
-    TMapPdfiumPages pages;
-    FPDF_FILEACCESS *psFileAccess;
-} TPdfiumDocumentStruct;
+    char *filename = nullptr;
+    CPDF_Document *doc = nullptr;
+    TMapPdfiumPages pages{};
+    FPDF_FILEACCESS *psFileAccess = nullptr;
+};
 
 #endif  // ~ HAVE_PDFIUM
 
@@ -266,6 +253,8 @@ class PDFDataset final : public GDALPamDataset
     void FindXMP(GDALPDFObject *poObj);
     void ParseInfo(GDALPDFObject *poObj);
 
+    CPL_DISALLOW_COPY_ASSIGN(PDFDataset)
+
 #ifdef HAVE_POPPLER
     std::unique_ptr<Object> m_poCatalogObjectPoppler{};
 #endif
@@ -360,8 +349,7 @@ class PDFDataset final : public GDALPamDataset
                                  const char *pszDefaultVal);
 
     bool m_bHasLoadedLayers = false;
-    int m_nLayers = 0;
-    OGRLayer **m_papoLayers = nullptr;
+    std::vector<std::unique_ptr<OGRPDFLayer>> m_apoLayers{};
 
     double m_dfPageWidth = 0;
     double m_dfPageHeight = 0;
@@ -383,7 +371,8 @@ class PDFDataset final : public GDALPamDataset
 
     void ExploreContentsNonStructuredInternal(
         GDALPDFObject *poContents, GDALPDFObject *poResources,
-        std::map<CPLString, OGRPDFLayer *> &oMapPropertyToLayer,
+        const std::map<CPLString, OGRPDFLayer *> &oMapPropertyToLayer,
+        const std::map<std::pair<int, int>, OGRPDFLayer *> &oMapNumGenToLayer,
         OGRPDFLayer *poSingleLayer);
     void ExploreContentsNonStructured(GDALPDFObject *poObj,
                                       GDALPDFObject *poResources);
@@ -391,15 +380,28 @@ class PDFDataset final : public GDALPamDataset
     int UnstackTokens(const char *pszToken, int nRequiredArgs,
                       char aszTokenStack[TOKEN_STACK_SIZE][MAX_TOKEN_SIZE],
                       int &nTokenStackSize, double *adfCoords);
-    OGRGeometry *
-    ParseContent(const char *pszContent, GDALPDFObject *poResources,
-                 int bInitBDCStack, int bMatchQ,
-                 std::map<CPLString, OGRPDFLayer *> &oMapPropertyToLayer,
-                 OGRPDFLayer *poCurLayer);
+
+    struct GraphicState
+    {
+        std::array<double, 6> adfCM = {1, 0, 0, 1, 0, 0};
+        std::array<double, 3> adfStrokeColor = {0.0, 0.0, 0.0};
+        std::array<double, 3> adfFillColor = {1.0, 1.0, 1.0};
+
+        void PreMultiplyBy(double adfMatrix[6]);
+        void ApplyMatrix(double adfCoords[2]) const;
+    };
+
+    OGRGeometry *ParseContent(
+        const char *pszContent, GDALPDFObject *poResources,
+        bool bCollectAllObjects, bool bInitBDCStack, bool bMatchQ,
+        const std::map<CPLString, OGRPDFLayer *> &oMapPropertyToLayer,
+        const std::map<std::pair<int, int>, OGRPDFLayer *> &oMapNumGenToLayer,
+        const GraphicState &graphicStateIn, OGRPDFLayer *poCurLayer,
+        int nRecLevel);
     OGRGeometry *BuildGeometry(std::vector<double> &oCoords, int bHasFoundFill,
                                int bHasMultiPart);
 
-    int OpenVectorLayers(GDALPDFDictionary *poPageDict);
+    bool OpenVectorLayers(GDALPDFDictionary *poPageDict);
 
     void InitOverviews();
 
@@ -425,8 +427,9 @@ class PDFDataset final : public GDALPamDataset
                                    const char *pszDomain = "") override;
 
     virtual CPLErr IRasterIO(GDALRWFlag, int, int, int, int, void *, int, int,
-                             GDALDataType, int, int *, GSpacing nPixelSpace,
-                             GSpacing nLineSpace, GSpacing nBandSpace,
+                             GDALDataType, int, BANDMAP_TYPE,
+                             GSpacing nPixelSpace, GSpacing nLineSpace,
+                             GSpacing nBandSpace,
                              GDALRasterIOExtraArg *psExtraArg) override;
 
     virtual int GetGCPCount() override;
@@ -525,6 +528,8 @@ class PDFWritableVectorDataset final : public GDALDataset
 
     int bModified;
 
+    CPL_DISALLOW_COPY_ASSIGN(PDFWritableVectorDataset)
+
   public:
     PDFWritableVectorDataset();
     virtual ~PDFWritableVectorDataset();
