@@ -7,29 +7,14 @@
  ******************************************************************************
  * Copyright (c) 2014, Even Rouault <even.rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
 #include "cpl_multiproc.h"
 #include "ograpispy.h"
 
+#include <cassert>
 #include <cstdio>
 #include <map>
 #include <set>
@@ -215,6 +200,7 @@ static bool OGRAPISpyEnabled()
     if (fpSpyFile == nullptr)
         fpSpyFile = stderr;
 
+    assert(fpSpyFile != nullptr);
     fprintf(fpSpyFile,
             "# This file is generated by the OGR_API_SPY mechanism.\n");
     fprintf(fpSpyFile, "import os\n");
@@ -490,7 +476,7 @@ int OGRAPISpyOpenTakeSnapshot(const char *pszName, int bUpdate)
             CPLString osWorkingDir;
             while (true)
             {
-                osBaseDir = CPLFormFilename(
+                osBaseDir = CPLFormFilenameSafe(
                     osSnapshotPath, CPLSPrintf("snapshot_%d", i), nullptr);
                 if (VSIStatL(osBaseDir, &sStat) != 0)
                     break;
@@ -498,9 +484,9 @@ int OGRAPISpyOpenTakeSnapshot(const char *pszName, int bUpdate)
             }
             VSIMkdir(osSnapshotPath, 0777);
             VSIMkdir(osBaseDir, 0777);
-            osSrcDir = CPLFormFilename(osBaseDir, "source", nullptr);
+            osSrcDir = CPLFormFilenameSafe(osBaseDir, "source", nullptr);
             VSIMkdir(osSrcDir, 0777);
-            osWorkingDir = CPLFormFilename(osBaseDir, "working", nullptr);
+            osWorkingDir = CPLFormFilenameSafe(osBaseDir, "working", nullptr);
             VSIMkdir(osWorkingDir, 0777);
 
             OGRAPISpyFileReopen();
@@ -513,9 +499,9 @@ int OGRAPISpyOpenTakeSnapshot(const char *pszName, int bUpdate)
             fprintf(fpSpyFile, "os.mkdir('%s')\n", osWorkingDir.c_str());
             for (char **papszIter = papszFileList; *papszIter; papszIter++)
             {
-                CPLString osSnapshotSrcFile = CPLFormFilename(
+                CPLString osSnapshotSrcFile = CPLFormFilenameSafe(
                     osSrcDir, CPLGetFilename(*papszIter), nullptr);
-                CPLString osSnapshotWorkingFile = CPLFormFilename(
+                CPLString osSnapshotWorkingFile = CPLFormFilenameSafe(
                     osWorkingDir, CPLGetFilename(*papszIter), nullptr);
                 CPL_IGNORE_RET_VAL(CPLCopyFile(osSnapshotSrcFile, *papszIter));
                 CPL_IGNORE_RET_VAL(
@@ -542,11 +528,12 @@ void OGRAPISpyOpen(const char *pszName, int bUpdate, int iSnapshot,
     CPLString osName;
     if (iSnapshot > 0)
     {
-        CPLString osBaseDir = CPLFormFilename(
+        CPLString osBaseDir = CPLFormFilenameSafe(
             osSnapshotPath, CPLSPrintf("snapshot_%d", iSnapshot), nullptr);
-        CPLString osWorkingDir = CPLFormFilename(osBaseDir, "working", nullptr);
+        CPLString osWorkingDir =
+            CPLFormFilenameSafe(osBaseDir, "working", nullptr);
         osName =
-            CPLFormFilename(osWorkingDir, CPLGetFilename(pszName), nullptr);
+            CPLFormFilenameSafe(osWorkingDir, CPLGetFilename(pszName), nullptr);
         pszName = osName.c_str();
 
         if (*phDS != nullptr)
@@ -612,8 +599,8 @@ void OGRAPISpyPostClose()
     }
 }
 
-void OGRAPISpyCreateDataSource(OGRSFDriverH hDriver, const char *pszName,
-                               char **papszOptions, OGRDataSourceH hDS)
+void OGRAPISpyCreateDataSource(GDALDriverH hDriver, const char *pszName,
+                               char **papszOptions, GDALDatasetH hDS)
 {
     if (!OGRAPISpyEnabled())
         return;
@@ -623,7 +610,7 @@ void OGRAPISpyCreateDataSource(OGRSFDriverH hDriver, const char *pszName,
         fprintf(fpSpyFile, "%s = ", OGRAPISpyGetDSVar(hDS).c_str());
     fprintf(fpSpyFile,
             "ogr.GetDriverByName('%s').CreateDataSource(%s, options=%s)\n",
-            GDALGetDriverShortName(reinterpret_cast<GDALDriverH>(hDriver)),
+            GDALGetDriverShortName(hDriver),
             OGRAPISpyGetString(pszName).c_str(),
             OGRAPISpyGetOptions(papszOptions).c_str());
     if (hDS != nullptr)
@@ -633,14 +620,14 @@ void OGRAPISpyCreateDataSource(OGRSFDriverH hDriver, const char *pszName,
     OGRAPISpyFileClose();
 }
 
-void OGRAPISpyDeleteDataSource(OGRSFDriverH hDriver, const char *pszName)
+void OGRAPISpyDeleteDataSource(GDALDriverH hDriver, const char *pszName)
 {
     if (!OGRAPISpyEnabled())
         return;
     CPLMutexHolderD(&hOGRAPISpyMutex);
     OGRAPISpyFlushDefered();
     fprintf(fpSpyFile, "ogr.GetDriverByName('%s').DeleteDataSource(%s)\n",
-            GDALGetDriverShortName(reinterpret_cast<GDALDriverH>(hDriver)),
+            GDALGetDriverShortName(hDriver),
             OGRAPISpyGetString(pszName).c_str());
     aoSetCreatedDS.erase(pszName);
     OGRAPISpyFileClose();
