@@ -10,23 +10,7 @@
  * Copyright (c) 2000, Frank Warmerdam
  * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 /* Some functions have been extracted from PostgreSQL code base  */
@@ -564,7 +548,7 @@ OGRFeature *OGRPGLayer::RecordToFeature(PGresult *hResult,
     for (int iField = 0; iField < PQnfields(hResult); iField++)
     {
 #if defined(BINARY_CURSOR_ENABLED)
-        int nTypeOID = PQftype(hResult, iField);
+        const Oid nTypeOID = PQftype(hResult, iField);
 #endif
         const char *pszFieldName = PQfname(hResult, iField);
 
@@ -1900,28 +1884,17 @@ const char *OGRPGLayer::GetFIDColumn()
 }
 
 /************************************************************************/
-/*                             GetExtent()                              */
+/*                            IGetExtent()                              */
 /*                                                                      */
 /*      For PostGIS use internal Extend(geometry) function              */
 /*      in other cases we use standard OGRLayer::GetExtent()            */
 /************************************************************************/
 
-OGRErr OGRPGLayer::GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
+OGRErr OGRPGLayer::IGetExtent(int iGeomField, OGREnvelope *psExtent,
+                              bool bForce)
 {
     CPLString osCommand;
 
-    if (iGeomField < 0 || iGeomField >= GetLayerDefn()->GetGeomFieldCount() ||
-        CPLAssertNotNull(GetLayerDefn()->GetGeomFieldDefn(iGeomField))
-                ->GetType() == wkbNone)
-    {
-        if (iGeomField != 0)
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                     "Invalid geometry field index : %d", iGeomField);
-        }
-        return OGRERR_FAILURE;
-    }
-
     OGRPGGeomFieldDefn *poGeomFieldDefn =
         poFeatureDefn->GetGeomFieldDefn(iGeomField);
 
@@ -1950,18 +1923,19 @@ OGRErr OGRPGLayer::GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce)
             OGRERR_NONE)
             return OGRERR_NONE;
     }
-    if (iGeomField == 0)
-        return OGRLayer::GetExtent(psExtent, bForce);
-    else
-        return OGRLayer::GetExtent(iGeomField, psExtent, bForce);
+
+    return OGRLayer::IGetExtent(iGeomField, psExtent, bForce);
 }
 
-OGRErr OGRPGLayer::GetExtent3D(int iGeomField, OGREnvelope3D *psExtent3D,
-                               int bForce)
+OGRErr OGRPGLayer::IGetExtent3D(int iGeomField, OGREnvelope3D *psExtent3D,
+                                bool bForce)
 {
+    auto poLayerDefn = GetLayerDefn();
+
     // If the geometry field is not 3D go for 2D
-    if (GetLayerDefn()->GetGeomFieldCount() > iGeomField &&
-        !OGR_GT_HasZ(GetLayerDefn()->GetGeomFieldDefn(iGeomField)->GetType()))
+    if (poLayerDefn->GetGeomFieldCount() > iGeomField &&
+        !OGR_GT_HasZ(CPLAssertNotNull(poLayerDefn->GetGeomFieldDefn(iGeomField))
+                         ->GetType()))
     {
         const OGRErr retVal{GetExtent(iGeomField, psExtent3D, bForce)};
         psExtent3D->MinZ = std::numeric_limits<double>::infinity();
@@ -1971,20 +1945,8 @@ OGRErr OGRPGLayer::GetExtent3D(int iGeomField, OGREnvelope3D *psExtent3D,
 
     CPLString osCommand;
 
-    if (iGeomField < 0 || iGeomField >= GetLayerDefn()->GetGeomFieldCount() ||
-        CPLAssertNotNull(GetLayerDefn()->GetGeomFieldDefn(iGeomField))
-                ->GetType() == wkbNone)
-    {
-        if (iGeomField != 0)
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                     "Invalid geometry field index : %d", iGeomField);
-        }
-        return OGRERR_FAILURE;
-    }
-
     OGRPGGeomFieldDefn *poGeomFieldDefn =
-        poFeatureDefn->GetGeomFieldDefn(iGeomField);
+        poLayerDefn->GetGeomFieldDefn(iGeomField);
 
     if (TestCapability(OLCFastGetExtent3D))
     {
@@ -2011,7 +1973,7 @@ OGRErr OGRPGLayer::GetExtent3D(int iGeomField, OGREnvelope3D *psExtent3D,
             return OGRERR_NONE;
     }
 
-    return OGRLayer::GetExtent3D(iGeomField, psExtent3D, bForce);
+    return OGRLayer::IGetExtent3D(iGeomField, psExtent3D, bForce);
 }
 
 /************************************************************************/
