@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2010-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include <algorithm>
@@ -135,8 +119,9 @@ char *OGRPGCommonLaunderName(const char *pszSrcName, const char *pszDebugPrefix,
     char *pszSafeName = bUTF8ToASCII ? CPLUTF8ForceToASCII(pszSrcName, '_')
                                      : CPLStrdup(pszSrcName);
 
-    int i = 0;  // needed after loop
-    for (; i < OGR_PG_NAMEDATALEN - 1 && pszSafeName[i] != '\0'; i++)
+    size_t i = 0;  // needed after loop
+    int iUTF8Char = 0;
+    for (; pszSafeName[i] != '\0'; i++)
     {
         if (static_cast<unsigned char>(pszSafeName[i]) <= 127)
         {
@@ -148,21 +133,44 @@ char *OGRPGCommonLaunderName(const char *pszSrcName, const char *pszDebugPrefix,
                 pszSafeName[i] = '_';
             }
         }
+
+        // Truncate string by making sure we don't cut in the
+        // middle of a UTF-8 multibyte character
+        // Continuation bytes of such characters are of the form
+        // 10xxxxxx (0x80), whereas single-byte are 0xxxxxxx
+        // and the start of a multi-byte is 11xxxxxx
+        if ((pszSafeName[i] & 0xc0) != 0x80)
+        {
+            ++iUTF8Char;
+            if (iUTF8Char == OGR_PG_NAMEDATALEN)
+                break;
+        }
     }
 
-    if (i == OGR_PG_NAMEDATALEN - 1 && pszSafeName[i] != '\0')
+    if (iUTF8Char == OGR_PG_NAMEDATALEN && pszSafeName[i] != '\0')
     {
         constexpr int FIRST_8_CHARS_OF_MD5 = 8;
-        pszSafeName[i - FIRST_8_CHARS_OF_MD5 - 1] = '_';
-        memcpy(pszSafeName + i - FIRST_8_CHARS_OF_MD5, CPLMD5String(pszSrcName),
+        iUTF8Char = 0;
+        for (i = 0; pszSafeName[i]; ++i)
+        {
+            if ((pszSafeName[i] & 0xc0) != 0x80)
+            {
+                ++iUTF8Char;
+                if (iUTF8Char == OGR_PG_NAMEDATALEN - FIRST_8_CHARS_OF_MD5 - 1)
+                    break;
+            }
+        }
+        pszSafeName[i] = '_';
+        memcpy(pszSafeName + i + 1, CPLMD5String(pszSrcName),
                FIRST_8_CHARS_OF_MD5);
+        i += FIRST_8_CHARS_OF_MD5 + 1;
     }
 
     pszSafeName[i] = '\0';
 
     if (strcmp(pszSrcName, pszSafeName) != 0)
     {
-        if (CPLStrlenUTF8(pszSafeName) < CPLStrlenUTF8(pszSrcName))
+        if (CPLStrlenUTF8Ex(pszSafeName) < CPLStrlenUTF8Ex(pszSrcName))
         {
             CPLError(CE_Warning, CPLE_AppDefined,
                      "%s identifier truncated to %s", pszSrcName, pszSafeName);
@@ -509,10 +517,21 @@ OGRPGDumpDataSource::ICreateLayer(const char *pszLayerName,
     if (bCreateTable && !osFIDColumnName.empty())
     {
         std::string osConstraintName(osTable);
-        if (osConstraintName.size() + strlen("_pk") >
+        if (CPLStrlenUTF8Ex(osTable.c_str()) + strlen("_pk") >
             static_cast<size_t>(OGR_PG_NAMEDATALEN - 1))
         {
-            osConstraintName.resize(OGR_PG_NAMEDATALEN - 1 - strlen("_pk"));
+            osConstraintName.clear();
+            size_t iUTF8Char = 0;
+            for (size_t i = 0; i < osTable.size(); ++i)
+            {
+                if ((osTable[i] & 0xc0) != 0x80)
+                {
+                    ++iUTF8Char;
+                    if (iUTF8Char == OGR_PG_NAMEDATALEN - strlen("_pk"))
+                        break;
+                }
+                osConstraintName += osTable[i];
+            }
         }
         osConstraintName += "_pk";
         osCommand.Printf(
