@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2011-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -132,14 +116,12 @@ OGRSQLiteLayer *OGRSQLiteViewLayer::GetUnderlyingLayer()
             osNewUnderlyingTableName.Printf(
                 "%s(%s)", m_osUnderlyingTableName.c_str(),
                 m_osUnderlyingGeometryColumn.c_str());
-            m_poUnderlyingLayer =
-                (OGRSQLiteLayer *)m_poDS->GetLayerByNameNotVisible(
-                    osNewUnderlyingTableName);
+            m_poUnderlyingLayer = cpl::down_cast<OGRSQLiteLayer *>(
+                m_poDS->GetLayerByNameNotVisible(osNewUnderlyingTableName));
         }
         if (m_poUnderlyingLayer == nullptr)
-            m_poUnderlyingLayer =
-                (OGRSQLiteLayer *)m_poDS->GetLayerByNameNotVisible(
-                    m_osUnderlyingTableName);
+            m_poUnderlyingLayer = cpl::down_cast<OGRSQLiteLayer *>(
+                m_poDS->GetLayerByNameNotVisible(m_osUnderlyingTableName));
     }
     return m_poUnderlyingLayer;
 }
@@ -331,9 +313,9 @@ OGRFeature *OGRSQLiteViewLayer::GetFeature(GIntBig nFeatureId)
 
     m_iNextShapeId = nFeatureId;
 
-    osSQL.Printf("SELECT \"%s\", * FROM '%s' WHERE \"%s\" = %d",
+    osSQL.Printf("SELECT \"%s\", * FROM '%s' WHERE \"%s\" = " CPL_FRMT_GIB,
                  SQLEscapeName(m_pszFIDColumn).c_str(), m_pszEscapedTableName,
-                 SQLEscapeName(m_pszFIDColumn).c_str(), (int)nFeatureId);
+                 SQLEscapeName(m_pszFIDColumn).c_str(), nFeatureId);
 
     CPLDebug("OGR_SQLITE", "exec(%s)", osSQL.c_str());
 
@@ -378,10 +360,10 @@ OGRErr OGRSQLiteViewLayer::SetAttributeFilter(const char *pszQuery)
 }
 
 /************************************************************************/
-/*                          SetSpatialFilter()                          */
+/*                         ISetSpatialFilter()                          */
 /************************************************************************/
 
-void OGRSQLiteViewLayer::SetSpatialFilter(OGRGeometry *poGeomIn)
+OGRErr OGRSQLiteViewLayer::ISetSpatialFilter(int, const OGRGeometry *poGeomIn)
 
 {
     if (InstallFilter(poGeomIn))
@@ -390,6 +372,7 @@ void OGRSQLiteViewLayer::SetSpatialFilter(OGRGeometry *poGeomIn)
 
         ResetReading();
     }
+    return OGRERR_NONE;
 }
 
 /************************************************************************/
